#!/usr/bin/perl
#
#  Wrapper script for searching with htdig.
#
#  This file is part of the SuSE help system.
#
#  Copyright (C) 2002,2003 SuSE Linux AG, Nuernberg
#
#  Author: Cornelius Schumacher <cschum@suse.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;

use FindBin;
use lib $FindBin::Bin;

use Encode;

use susehelp;

use open IO => ':utf8';
use open ':std';

&checkQuery;
&parseQuery;

my $config = getQuery( "config" );
my $format = getQuery( "format" );
my $method = getQuery( "method" );
my $words = getQuery( "words" );
my $lang = getQuery( "lang" );

my $charset = langCharset( $lang );

$words = encode( $charset, $words );

my $indexdir = indexDir();

if ( !$indexdir ) { $indexdir = "/var/cache/susehelp"; }

$config =~ s/\.//g;
$config =~ s/\-//g;

my $htsearchcmd = "$susehelp::htsearchpath -c $indexdir/$config.conf ";
$htsearchcmd .= "\"format=$format&method=$method&words=$words\"";

delete $ENV{ "SERVER_PROTOCOL" };
delete $ENV{ "REQUEST_METHOD" };
delete $ENV{ "QUERY_STRING" };

&dbg( "HTSEARCHCMD: $htsearchcmd\n" );

if ( !open( HTSEARCH, "$htsearchcmd|" ) ) {
  print "Can't execute htsearch at '$susehelp::htsearchpath'.\n";
  exit 1;
}

my ($body,$liststart,$ref,$link,$error,$errorOut);

while( <HTSEARCH> ) {
  if ( !$body ) {
    print;
    if ( /^<body/ ) { $body = 1; }
  }
  if ( /^<h3>/ ) {
    print;
    print "<ul>\n";
    $liststart = 1;
  }
  if ( /^<img src.*<a href="(.*)">(.*)<\/a>/ ) {
    $ref = $1;
    $link = $2;
    $ref =~ s/file:\/\/localhost//;
    
    #Special treatment for KDE docs
    if ( $ref =~ /\/([^\/]*)\/index\.docbook$/ ) {
      $ref = "help:$1";
    }

    $ref =~ s/http:\/\/localhost//;
    
    print "  <li><a href=\"$ref\">$link</a></li>\n";
  }
  if ( /^<h1>ht:\/\/Dig error/ ) {
    $error = 1;
    print "Htdig error:\n";
  }
  if ( $error && /^<pre>/ ) {
    $errorOut = 1;
  }
  if ( $errorOut ) {
    print;
    if ( /^<\/pre>/ ) { $errorOut = 0; }
  }
}

close HTSEARCH;

if ( $liststart ) { print "</ul>\n"; }

print "</body></html>\n";

if ( $? != 0 ) { exit $?; }
