#!/usr/bin/perl
#
#  Script for displaying a man page.
#
#  This file is part of the SuSE help system.
#
#  Copyright (C) 2002  SuSE Linux AG, Nuernberg
#
#  Author: Cornelius Schumacher <cschum@suse.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use FindBin;
use lib $FindBin::Bin;

use substitution;
use susehelp;

&checkQuery();
&parseQuery();

my $language = lc getQuery( "lang", "en" );
my $page = getQuery( "page" );
my $section = getQuery( "section" );

if ( !$section ) {
  $section = "";
}

if ( !$page ) {
  printErrorPage( "Please provide an argument." );
}

$page =~ s/([^a-zA-Z0-9])//g;
$section =~ s/([^a-zA-Z0-9])//g;

my $cgi = createCgiUrl( "manshow.cgi" );
my $cgiref = $cgi . "&page=%s&section=%s";

if ( system( "man -w $section $page > /dev/null" ) != 0 ) {
  printErrorPage( "Can't open manpage" );
}

if ( !open MAN, "man $section $page | rman -f html -r '$cgiref'| recode latin1..utf8|" ) {
  printErrorPage( "Can't open manpage" );
}

&printCgiHeader;

$out = "--HEADER:--";

while( <MAN> ) {
  if ( /<BODY/i ) { $body = 1; next; }
  if ( /<\/BODY/i ) { last; }
  if ( $body ) { $out .= $_; }
}

$out .= "--FOOTER:--";

printPage( $out );
