#!/usr/bin/perl
#
#  This script generates a set of HTML files with links to package descriptions
#  for all available packages.
#
#  This file is part of the SuSE help system.
#
#  Copyright (C) 1994-2001  SuSE GmbH
#  Copyright (C) 2002  SuSE Linux AG, Nuernberg
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#use strict;
#use utf8;

do "/usr/share/susehelp/susehelprc";

push @INC, $susehelpcgidir;

require substitution;
import substitution;

require susehelp;
import susehelp;

use constant DESCRPATH => "/var/adm/current_package_descr/suse/setup/descr/";

$paclistdir = "/var/lib/susehelp/pac";

sub makeHeader($);
sub cleanText($);
sub getNext(*$);
sub finishFile($);
sub addLink($$);

my $language;

if ( @ARGV == 1 ) {
  $language = lc( $ARGV[ 0 ] );
} else {
  print STDERR "Usage: $0 <language>\n";
  exit 1;
}

setLanguage($language,"pac.po");

$paclistdir .= "/$language";

system( "mkdir -p $paclistdir" );

#
# now open language specific pkd file and common pkd file
unless (open(PKDLOCALE, DESCRPATH . $language . ".pkd")) {
  open(PKDLOCALE, DESCRPATH . "en.pkd") or die "could not even open en.pkd (as fallback)";
}
open(PKDCOMMON, DESCRPATH . "common.pkd") or die "could not open common.pkd";

# generate Hash of all packages and their RPM Groups
my %packagelinks;
my %allpacklinks;
my $current;

while (<PKDLOCALE>) {

  next unless (/^Filename:\s*(.*)\.(.*?)\.rpm/);
  $current = $1;
  my $arch = $2;
  $current =~ s/\+/\\\+/;

  if ( $arch =~ "src" ) { next; }
  
  while (<PKDCOMMON>) {
    last if /^Filename:\s*$current\.$arch\.rpm/
  }

  my $rpmname = getNext(PKDCOMMON,"RpmName:");

  my $linktocurrent = "<li><a href=\"cgi:pacshow.cgi?lang=$language&package=$rpmname\">";

  $linktocurrent .= "<b>$rpmname</b> - ";

  $allpacklinks{$rpmname} .= $linktocurrent . getNext(PKDCOMMON, "Series:") .
                             " - " . getNext(PKDCOMMON, "RpmGroup:") .
                             "</a></li>\n";

  $linktocurrent .= getNext(PKDLOCALE, "Label:") . "</a></li>";

  my $RPMgroup = getNext(PKDCOMMON, "RpmGroup:");
  $RPMgroup =~ s/\s/-/g;
  $packagelinks{$RPMgroup . "/$rpmname"} = $linktocurrent;

}

close(PKDLOCALE) or die "could not close " . $language . ".pkd";
close(PKDCOMMON) or die "could not close common.pkd";

# start building directory structure

my @files;

#makeHeader("index");
#push @files, "index";

# generate page for all packages

#makeHeader("allpacks");
#push @files, "allpacks";
#my $allpacksfile = "$paclistdir/allpacks.html";
#open(ALLPACKS, ">>$allpacksfile") or die "could not open $allpacksfile";
#print ALLPACKS $allpacklinks{$_} foreach sort keys %allpacklinks;
#close(ALLPACKS);

my %groups;
my %packages;

foreach (sort keys %packagelinks) {

  /(.*)\/(.*?)$/;

  my $group = $1;
  my $package = $2;

  push @{ $packages{ $group } }, $package;
}

readSymbols( $susehelp::susehelpsymbols );
registerSymbol( "PACLIST_TITLE", susehelp::i18n('Package descriptions') );
registerSymbol( "PACLIST_CHANGED", localtime() );

my %parentgroups;

#foreach ( sort keys %packagelinks ) { print "$_\n"; }

# Generate leaf node files.
foreach $group ( sort keys %packages ) {
#  print "$group\n";

  # Create hash of parent groups for later processing
  my @groupparts = split /\//, $group;
  my $groupkey = "";
  while ( @groupparts ) {
    $g = shift @groupparts;
    $parentgroups { $groupkey }{ $g } = 1;
    if ( $groupkey ) { $groupkey .= "_"; }
    $groupkey .= $g;
  }

  my $out = "--HEADER:--";

  $out .= "--PACLIST_TOP:--";

  registerSymbol( "PACLIST_HELLO", generateGroupLinks( $groupkey ) );

  foreach $package ( sort @{ $packages{ $group } } ) {
#    print "  $package\n";
    $out .= $packagelinks{ "$group/$package" };
  }

  $out .= "--PACLIST_BOTTOM:--";

  writeFile( $groupkey, $out );
}

sub generateGroupLinks($)
{
  my $groupkey = shift;

  my $nicename = "<a href=\"$paclistdir/index.html\">index</a>/";
  my $currentURL = "";  
  foreach (split "_", $groupkey) {
    $currentURL .= $_;
    $nicename .= "<a href=\"$paclistdir/$currentURL.html\">$_</a>/";
    $currentURL .= "_";
  }

  return $nicename;
}

# Generate parent group files.
foreach $groupkey ( sort keys %parentgroups ) {

  my $out = "--HEADER:--";

  $out .= "--PACLIST_TOP:--";

  registerSymbol( "PACLIST_HELLO", generateGroupLinks( $groupkey ) );

  # Generate child group links
  foreach $childgroup ( sort keys %{ $parentgroups{ $groupkey } } ) {
    my $link = $groupkey;
    if ( $link ) { $link .= "_"; }
    $link .= $childgroup . ".html";
    $out .= "<li><a href=\"$paclistdir/$link\">$childgroup</a></li>\n";
  }

  $out .= "--PACLIST_BOTTOM:--";

  writeFile( $groupkey, $out );
}

exit 0;

sub writeFile( $$ )
{
  my $groupkey = shift;
  my $out = shift;
  
  my $filename = "$paclistdir/";
  if ( $groupkey ) { $filename .= $groupkey }
  else { $filename .= "index"; }
  $filename .= ".html";
  
  open ( OUT , ">$filename" ) or die "could not open $filename";
  print OUT substituteSymbols( $out );
  close ( OUT ) or die "could not close $filename";  
}  

sub nop()
{
  my $combinedparts = "";
  my $lastcombination = "";

  # check each element
  my $currpart;
  while (@keyparts) {
    $currpart = shift @keyparts;

    # recombine parts to generate files
    $combinedparts .= "_" if $combinedparts;
    $combinedparts .= $currpart;

    if (@keyparts) {
#      print STDERR "Check $paclistdir/$lastcombination.html\n";
      unless (-e "$paclistdir/$lastcombination.html") {
        makeHeader( "$lastcombination" );
        #remember file names, cause we need to close them later
        push @files, $combinedparts;
        addLink($combinedparts, $lastcombination);
      }
    } else {
      my $lastcombinationfile = "$paclistdir/$lastcombination.html";
      open(GROUPPAGE, ">>$lastcombinationfile") or die "could not open $lastcombinationfile";
      print GROUPPAGE $packagelinks{$currkey} . "\n";
      close(GROUPPAGE) or die "could not close $lastcombinationfile";
    }
    $lastcombination = $combinedparts;
  }
}

finishFile($_) foreach @files;

#
# THE END
#


# makeHeader
#
# this function adds the opening html tags to a newly created file
sub makeHeader($) {
  my $filename = shift;

  my $nicename = "";
  if ($filename eq "allpacks") {
     $nicename = susehelp::i18n('Summary');
  }
  else {
     $nicename = "<a href=\"index.html\">index</a>/" unless $filename eq "index";
     my $currentURL = "";
     foreach (split "_", $filename)
     {
       $currentURL .= $_;
       $nicename .= "<a href=\"$currentURL.html\">$_</a>/";
       $currentURL .= "_";
     }
  }

  my $out = "--HEADER:--";

  $out .= "--PACLIST_TOP:--";

  readSymbols( $susehelp::susehelpsymbols );
  registerSymbol( "PACLIST_TITLE", susehelp::i18n('Package descriptions') );
  registerSymbol( "PACLIST_HELLO", $nicename );

  $filename = "$paclistdir/$filename";
  open(HTMLFILE, ">$filename.html") or die "could not open $filename";
  print HTMLFILE substituteSymbols( $out );
  close(HTMLFILE) or die "could not close $filename";
}

# addLink
#
# this function adds a list element (link) to a file
sub addLink($$) {
  my $link = shift;
  my $filename = shift;
  $filename = "index" unless $filename;

  $shortlink = $link;
  $shortlink =~ s/.*_//g;

#  print STDERR "ADDLINK: $filename  $link\n";

  $filename = "$paclistdir/$filename";
  open(HTMLFILE, ">>$filename.html") or die "could not open $filename";
  print HTMLFILE "<li><a href=\"./$link.html\">$shortlink</a>\n";
  close(HTMLFILE) or die "could not close $filename";
}


# finishFile
#
# this function finishes a file by appending the closing html tags
sub finishFile($) {
  my $filename = shift;
  my $nicename = "";
  
  if ($filename eq "allpacks") {
     $nicename = "<a href=\"index.html\">index</a></h3>";
  }
  else {
     $nicename = "<a href=\"allpacks.html\">" . susehelp::i18n('Summary') . "</a>";
  }

  my $out = "--PACLIST_BOTTOM:--";
  
  registerSymbol( "PACLIST_CHANGED", localtime() );

  $filename = "$paclistdir/$filename";
  open(FINISHFILE, ">>$filename.html") or die "could not open $filename";
  print FINISHFILE substituteSymbols( $out );
  close(FINISHFILE) or die "could not close $filename";
}


#
# cleanText
# changes some problems with special characters in HTML
sub cleanText($) {
  my $text = shift;
  $text =~ s/</&lt;/g;
  $text =~ s/>/&gt;/g;
  $text =~ s/\\n/<BR>/g;
  return $text;
}

# getNext
# return value of next entry with the given name
#
sub getNext(*$) {
  my $filehandle = shift;
  my $word = shift;

  my $pos = tell $filehandle;

  while (<$filehandle>) {
    if ($_ =~ /$word\s*(.*?)\s*$/i) {
      seek($filehandle, $pos, 0);
      return ($1) ? cleanText($1) : "";
    }
  }
  die "could not find $word";
}

